@@ -3,9 +3,10 @@
 
 import re
 from . import utils
-from .msg import err_buf
+from . import log
 
 
+logger = log.setup_logger(__name__)
 cib_verify = "crm_verify -VV -p"
 VALIDATE_RE = re.compile(r"^Entity: line (\d)+: element (\w+): " +
                          r"Relax-NG validity error : (.+)$")
@@ -23,9 +24,9 @@ def verify(cib):
     for i, line in enumerate(line for line in stderr.split('\n') if line):
         if i == 0:
             if "warning:" in line:
-                err_buf.warning(_prettify(line, 0))
+                logger.warning(_prettify(line, 0))
             else:
-                err_buf.error(_prettify(line, 0))
+                logger.error(_prettify(line, 0))
         else:
-            err_buf.writemsg(_prettify(line, 7))
+            logger.error(_prettify(line, 7))
     return rc
