@@ -3,17 +3,16 @@ import sys
 import re
 import time
 import threading
-import logging
 import shutil
 import tempfile
 from contextlib import contextmanager
 from crmsh import utils as crmshutils
-
+from crmsh import log
 from . import utils
 from . import config
 
 
-logger = logging.getLogger('cpc')
+logger = log.setup_logger(__name__)
 
 
 class TaskError(Exception):
