@@ -11,8 +11,13 @@ from .ra import disambiguate_ra_type, ra_type_validate
 from . import schema
 from .utils import keyword_cmp, verify_boolean, lines2cli
 from .utils import get_boolean, olist, canonical_boolean
-from .msg import common_err, syntax_err
+from .utils import handle_role_for_ocf_1_1
 from . import xmlutil
+from . import log
+
+
+logger = log.setup_logger(__name__)
+logger_utils = log.LoggerUtils(logger)
 
 
 _NVPAIR_RE = re.compile(r'([^=@$][^=]*)=(.*)$')
@@ -146,7 +151,7 @@ class BaseParser(object):
             token = self._cmd[self._currtok]
         if context is None:
             context = self._cmd[0]
-        syntax_err(self._cmd, context=context, token=token, msg=msg)
+        logger_utils.syntax_err(self._cmd, context=context, token=token, msg=msg)
         raise ParseError
 
     def begin(self, cmd, min_args=-1):
@@ -164,11 +169,12 @@ class BaseParser(object):
         self.begin(cmd, min_args=min_args)
         return self.match_dispatch(errmsg="Unknown command")
 
-    def do_parse(self, cmd):
+    def do_parse(self, cmd, ignore_empty):
         """
         Called by CliParser. Calls parse()
         Parsers should pass their return value through this method.
         """
+        self.ignore_empty = ignore_empty
         out = self.parse(cmd)
         if self.has_tokens():
             self.err("Unknown arguments: " + ' '.join(self._cmd[self._currtok:]))
@@ -462,7 +468,7 @@ class BaseParser(object):
                     idref = True
                 rule.set(idtyp, idval)
             if self.try_match(_ROLE_RE):
-                rule.set('role', self.matched(1))
+                rule.set('role', handle_role_for_ocf_1_1(self.matched(1)))
             if idref:
                 continue
             if self.try_match(_SCORE_RE):
@@ -919,10 +925,13 @@ class ConstraintParser(BaseParser):
             out.set('rsc', self.match_resource())
 
         while self.try_match(_ATTR_RE):
-            out.set(self.matched(1), self.matched(2))
+            name = self.matched(1)
+            value = handle_role_for_ocf_1_1(self.matched(2), name=name)
+            out.set(name, value)
 
+        # not sure this is necessary after parse _ATTR_RE in a while loop
         if self.try_match(_ROLE_RE) or self.try_match(_ROLE2_RE):
-            out.set('role', self.matched(1))
+            out.set('role', handle_role_for_ocf_1_1(self.matched(1)))
 
         score = False
         if self.try_match(_SCORE_RE):
@@ -932,7 +941,7 @@ class ConstraintParser(BaseParser):
             # backwards compatibility: role used to be read here
             if 'role' not in out:
                 if self.try_match(_ROLE_RE) or self.try_match(_ROLE2_RE):
-                    out.set('role', self.matched(1))
+                    out.set('role', handle_role_for_ocf_1_1(self.matched(1)))
         if not score:
             rules = self.match_rules()
             out.extend(rules)
@@ -1035,7 +1044,7 @@ class ConstraintParser(BaseParser):
 
     def _split_setref(self, typename, classifier):
         rsc, typ = self.match_split()
-        typ, t = classifier(typ)
+        typ, t = classifier(handle_role_for_ocf_1_1(typ, name=typename))
         if typ and not t:
             self.err("Invalid %s '%s' for '%s'" % (typename, typ, rsc))
         return rsc, typ, t
@@ -1101,7 +1110,11 @@ def property_parser(self, cmd):
         attrs.set(idkey, idval)
     for rule in self.match_rules():
         attrs.append(rule)
-    for nvp in self.match_nvpairs(terminator=[], minpairs=0, allow_empty=False):
+    if self.ignore_empty:
+        res_list = self.match_nvpairs(minpairs=0)
+    else:
+        res_list = self.match_nvpairs(terminator=[], minpairs=0, allow_empty=False)
+    for nvp in res_list:
         attrs.append(nvp)
     return root
 
@@ -1447,7 +1460,7 @@ def parse_xml(self, cmd):
     try:
         e = etree.fromstring(xml_data)
     except Exception as err:
-        common_err("Cannot parse XML data: %s" % xml_data)
+        logger.error("Cannot parse XML data: %s" % xml_data)
         self.err(err)
     if e.tag not in constants.cib_cli_map:
         self.err("Element %s not recognized" % (e.tag))
@@ -1618,7 +1631,7 @@ class ResourceSet(object):
                     validator.resource_actions())
             else:
                 l[1] = validator.canonize(
-                    l[1],
+                    handle_role_for_ocf_1_1(l[1]),
                     validator.resource_roles())
             if not l[1]:
                 self.err('Invalid %s for %s' % (self.q_attr, p))
@@ -1690,7 +1703,7 @@ class ResourceSet(object):
         return ret
 
 
-def parse(s, comments=None):
+def parse(s, comments=None, ignore_empty=True):
     '''
     Input: a list of tokens (or a CLI format string).
     Return: a cibobject
@@ -1708,7 +1721,7 @@ def parse(s, comments=None):
             s = s.encode('ascii', errors='xmlcharrefreplace')
             s = s.decode('utf-8')
         except Exception as e:
-            common_err(e)
+            logger.error(e)
             return False
     if isinstance(s, str):
         if s and s.startswith('#'):
@@ -1718,7 +1731,7 @@ def parse(s, comments=None):
             try:
                 s = [x for p in lines2cli(s) for x in p.split()]
             except ValueError as e:
-                common_err(e)
+                logger.error(e)
                 return False
         else:
             s = shlex.split(s)
@@ -1732,11 +1745,11 @@ def parse(s, comments=None):
     kw = s[0]
     parser = _parsers.get(kw)
     if parser is None:
-        syntax_err(s, token=s[0], msg="Unknown command")
+        logger_utils.syntax_err(s, token=s[0], msg="Unknown command")
         return False
 
     try:
-        ret = parser.do_parse(s)
+        ret = parser.do_parse(s, ignore_empty)
         if ret is not None and len(comments) > 0:
             if ret.tag in constants.defaults_tags:
                 xmlutil.stuff_comments(ret[0], comments)
