@@ -16,8 +16,6 @@ from .cibconfig import mkset_obj, cib_factory
 from . import clidisplay
 from . import term
 from . import options
-from .msg import common_err, common_info, common_warn
-from .msg import err_buf, syntax_err
 from . import rsctest
 from . import schema
 from . import ui_cib
@@ -31,6 +29,11 @@ from .crm_gv import gv_types
 from .ui_node import get_resources_on_nodes, remove_redundant_attrs
 
 
+from . import log
+logger = log.setup_logger(__name__)
+logger_utils = log.LoggerUtils(logger)
+
+
 def _type_completions():
     "completer for type: use in show"
     typelist = cib_factory.type_list()
@@ -241,43 +244,47 @@ def _prim_meta_completer(agent, args):
 
 
 def _prim_op_completer(agent, args):
-    actions = agent.actions()
-    completing = args[-1]
-    if completing == 'op':
+
+    def concat_kv(k, v):
+        return "{}={}".format(k, v)
+
+    if args[-1] == 'op':
         return ['op']
+    actions = agent.actions()
+    if not actions:
+        return []
+    # list all actions, select one to complete
     if args[-2] == 'op':
-        # append action items which in agent default actions
-        # monitor_Master will be mapped to "monitor role=Master"
-        # monitor_Slave will be mapped to "monitor role=Slave"
-        op_list = list(constants.op_cli_names)
-        if "monitor_Master" in actions:
-            op_list.append("monitor_Master")
-        if "monitor_Slave" in actions:
-            op_list.append("monitor_Slave")
-        # remove action items which not in default actions
-        for item in ["monitor", "demote", "promote", "notify"]:
-            if item not in actions:
-                op_list.remove(item)
-        # remove action items which already used
-        for item in op_list:
-            if item in args[:-2]:
-                op_list.remove(item)
-        return op_list
+        return actions.keys()
+    # list all attributes of the action, select one to complete
     if args[-3] == 'op':
         res = []
-        # list all of default items
-        if actions and actions[args[-2]]:
-            for k, v in list(actions[args[-2]].items()):
-                res += ["%s=%s" % (k, v)]
-            return res
+        op_name = args[-2]
+        if op_name == 'monitor':
+            for one_monitor in actions[op_name]:
+                res += [concat_kv(k, v) for k, v in one_monitor.items()]
+        else:
+            res = [concat_kv(k, v) for k, v in actions[op_name].items()]
+        return res
+
     args.pop()
-    # make sure all of default items can be completed
-    if args[-2] in actions:
+    if '=' in args[-1]:
         res = []
-        for k, v in actions[args[-2]].items():
-            if args[-1].startswith(k+'='):
-                continue
-            res += ["%s=%s" % (k, v)]
+        # find latest action
+        op_name = None
+        for i, item in enumerate(reversed(args)):
+            if item in actions:
+                op_name = item
+                break
+        if not op_name:
+            return []
+        # list all left attributes of the action, select one to complete
+        actions_list_in_args = [arg.split('=')[0] for arg in args[len(args)-i:]]
+        if op_name == 'monitor':
+            for one_monitor in actions[op_name]:
+                res += [concat_kv(k, v) for k, v in one_monitor.items() if k not in actions_list_in_args]
+        else:
+            res = [concat_kv(k, v) for k, v in actions[op_name].items() if k not in actions_list_in_args]
         return res
 
     return []
@@ -328,7 +335,7 @@ def primitive_complete_complex(args):
         return []
 
     complete_results = completers_set[last_keyw](agent, args)
-    if len(args) > 4 and '=' in args[-2]: # args[-1] will be the space
+    if len(args) > 4 and '=' in args[-1]:
         return complete_results + keywords
 
     return complete_results
@@ -499,7 +506,7 @@ class CibConfig(command.UI):
         if not has_simulate:
             constants.simulate_programs["simulate"] = "ptest"
         if not (has_ptest or has_simulate):
-            common_warn("neither ptest nor crm_simulate exist, check your installation")
+            logger.warning("neither ptest nor crm_simulate exist, check your installation")
             constants.simulate_programs["ptest"] = ""
             constants.simulate_programs["simulate"] = ""
         return True
@@ -656,7 +663,7 @@ class CibConfig(command.UI):
         """usage: modgroup <id> add <id> [after <id>|before <id>]
         modgroup <id> remove <id>"""
         if subcmd not in ("add", "remove"):
-            common_err("modgroup subcommand %s unknown" % subcmd)
+            logger.error("modgroup subcommand %s unknown" % subcmd)
             return False
         after_before = None
         if args:
@@ -701,9 +708,8 @@ class CibConfig(command.UI):
     @command.completers_repeating(_id_xml_list, _id_list)
     def do_edit(self, context, *args):
         "usage: edit [xml] [<id>...]"
-        err_buf.buffer()  # keep error messages
-        set_obj = mkset_obj(*args)
-        err_buf.release()  # show them, but get an ack from the user
+        with logger_utils.buffer():  # keep error messages
+            set_obj = mkset_obj(*args)
         return set_obj.edit()
 
     def _verify(self, set_obj_semantic, set_obj_all):
@@ -873,8 +879,8 @@ class CibConfig(command.UI):
 
     def _commit(self, force=False, replace=False):
         if not cib_factory.has_cib_changed():
-            common_info("apparently there is nothing to commit")
-            common_info("try changing something first")
+            logger.info("apparently there is nothing to commit")
+            logger.info("try changing something first")
             return True
         replace = replace or not utils.cibadmin_can_patch()
         rc1 = True
@@ -885,7 +891,7 @@ class CibConfig(command.UI):
         if rc1 and rc2:
             return cib_factory.commit(replace=replace)
         if force or config.core.force:
-            common_info("commit forced")
+            logger.info("commit forced")
             return cib_factory.commit(force=True, replace=replace)
         if utils.ask("Do you still want to commit?"):
             return cib_factory.commit(force=True, replace=replace)
@@ -899,10 +905,10 @@ class CibConfig(command.UI):
         force = "force" in [arg0, arg1]
         replace = "replace" in [arg0, arg1]
         if arg0 is not None and arg0 not in ("force", "replace"):
-            syntax_err(('configure.commit', arg0))
+            logger_utils.syntax_err(('configure.commit', arg0))
             return False
         if arg1 is not None and arg1 not in ("force", "replace"):
-            syntax_err(('configure.commit', arg1))
+            logger_utils.syntax_err(('configure.commit', arg1))
             return False
         return self._commit(force=force, replace=replace)
 
@@ -954,7 +960,7 @@ class CibConfig(command.UI):
         "The configure object command."
         if cmd in list(constants.cib_cli_map.values()) and \
                 not cib_factory.is_elem_supported(cmd):
-            common_err("%s not supported by the RNG schema" % cmd)
+            logger.error("%s not supported by the RNG schema" % cmd)
             return False
         if not args:
             return cib_factory.create_object(cmd, *args)
@@ -985,14 +991,7 @@ class CibConfig(command.UI):
             [op op_type [<attribute>=<value>...]
                         [[op_params] <param>=<value> [<param>=<value>...]]
                         [op_meta <attribute>=<value> [<attribute>=<value>...]] ...]]"""
-        tmp = list(args)
-        for item in ['monitor_Master', 'monitor_Slave']:
-            if item in tmp:
-                idx = tmp.index(item)
-                tmp.remove(item)
-                tmp.insert(idx, "monitor")
-                tmp.insert(idx+1, "role=%s" % item.split('_')[1])
-        return self.__conf_object(context.get_command_name(), *tuple(tmp))
+        return self.__conf_object(context.get_command_name(), *args)
 
     @command.completers_repeating(compl.attr_id, _container_type, container_complete_complex)
     def do_bundle(self, context, *args):
@@ -1025,6 +1024,9 @@ class CibConfig(command.UI):
         """usage: ms <name> <rsc>
         [params <param>=<value> [<param>=<value>...]]
         [meta <attribute>=<value> [<attribute>=<value>...]]"""
+        format_str = " " if "meta" in args else " meta "
+        new_cmd_str = ' '.join(args) + "{}promotable=true".format(format_str)
+        logger.warning('"ms" is deprecated. Please use "clone {}"'.format(new_cmd_str))
         return self.__conf_object(context.get_command_name(), *args)
 
     @command.skill_level('administrator')
@@ -1181,7 +1183,7 @@ class CibConfig(command.UI):
         for ident in args:
             el = cib_factory.find_object(ident)
             if not el:
-                common_err("element %s does not exist" % ident)
+                logger.error("element %s does not exist" % ident)
                 rc = False
             elif current == "r" and xmlutil.is_resource(el.node):
                 if xmlutil.is_container(el.node):
@@ -1192,12 +1194,12 @@ class CibConfig(command.UI):
                 current = "n"
                 node_l.append(el.node.get("uname"))
             else:
-                syntax_err((context.get_command_name(), ident), context='rsctest')
+                logger_utils.syntax_err((context.get_command_name(), ident), context='rsctest')
                 return False
         if not rc:
             return False
         if not rsc_l:
-            common_err("specify at least one resource")
+            logger.error("specify at least one resource")
             return False
         all_nodes = cib_factory.node_id_list()
         if not node_l:
