@@ -2,6 +2,7 @@
 # Copyright (C) 2013 Kristoffer Gronlund <kgronlund@suse.com>
 # See COPYING for license information.
 
+import re
 from . import config
 from . import command
 from . import completers as compl
@@ -9,12 +10,17 @@ from . import constants
 from . import ui_utils
 from . import utils
 from . import xmlutil
-from .msg import common_err, syntax_err, no_prog_err, common_info, common_warn
-from .msg import common_debug
 from .cliformat import cli_nvpairs, nvpairs2list
 from . import term
 from .cibconfig import cib_factory
 from .ui_resource import rm_meta_attribute
+from .ui_cluster import parse_option_for_nodes
+from . import log
+
+
+logger = log.setup_logger(__name__)
+logger_utils = log.LoggerUtils(logger)
+
 
 def remove_redundant_attrs(objs, attributes_tag, attr, conflicting_attr = None):
     """
@@ -34,8 +40,7 @@ def remove_redundant_attrs(objs, attributes_tag, attr, conflicting_attr = None):
                 (config.core.manage_children == "ask" and
                 utils.ask("'%s' attribute already exists in %s. Remove it?" %
                         (attr, r.get(field2show))))):
-                common_debug("force remove meta attr %s from %s" %
-                        (attr, r.get(field2show)))
+                logger.debug("force remove meta attr %s from %s", attr, r.get(field2show))
                 xmlutil.rmnode(a)
                 xmlutil.xml_processnodes(r, xmlutil.is_emptynvpairs, xmlutil.rmnodes)
             if conflicting_attr is not None:
@@ -45,8 +50,7 @@ def remove_redundant_attrs(objs, attributes_tag, attr, conflicting_attr = None):
                     (config.core.manage_children == "ask" and
                     utils.ask("'%s' conflicts with '%s' in %s. Remove it?" %
                             (conflicting_attr, attr, r.get(field2show))))):
-                    common_debug("force remove meta attr %s from %s" %
-                            (conflicting_attr, r.get(field2show)))
+                    logger.debug("force remove meta attr %s from %s", conflicting_attr, r.get(field2show))
                     xmlutil.rmnode(a)
                     xmlutil.xml_processnodes(r, xmlutil.is_emptynvpairs, xmlutil.rmnodes)
 
@@ -71,10 +75,10 @@ def update_xml_node(cluster_node_name, attr, value):
 
     node_obj = cib_factory.find_node(cluster_node_name)
     if node_obj is None:
-        common_err("CIB is not valid!")
+        logger.error("CIB is not valid!")
         return False
 
-    common_debug("update_xml_node: %s" % (node_obj.obj_id))
+    logger.debug("update_xml_node: %s", node_obj.obj_id)
 
     xml_node = node_obj.node
     node_obj.set_updated()
@@ -115,14 +119,14 @@ def set_node_attr(cluster_node_name, attr_name, value, commit=True):
     Set an attribute for a node
     """
     if not update_xml_node(cluster_node_name, attr_name, value):
-        common_err("Failed to update node attributes for %s" % (cluster_node_name))
+        logger.error("Failed to update node attributes for %s", cluster_node_name)
         return False
 
     if not commit:
         return True
 
     if not cib_factory.commit():
-        common_err("Failed to commit updates to %s" % (cluster_node_name))
+        logger.error("Failed to commit updates to %s", cluster_node_name)
         return False
     return True
 
@@ -246,7 +250,7 @@ class NodeMgmt(command.UI):
     def requires(self):
         for p in ('cibadmin', 'crm_attribute'):
             if not utils.is_program(p):
-                no_prog_err(p)
+                logger_utils.no_prog_err(p)
                 return False
         return True
 
@@ -285,36 +289,87 @@ class NodeMgmt(command.UI):
     @command.wait
     @command.completers(compl.nodes)
     def do_standby(self, context, *args):
-        'usage: standby [<node>] [<lifetime>]'
-        argl = list(args)
-        node = None
-        lifetime = utils.fetch_lifetime_opt(argl, iso8601=False)
-        if not argl:
-            node = utils.this_node()
-        elif len(argl) == 1:
-            node = args[0]
-            if not xmlutil.is_our_node(node):
-                common_err("%s: node name not recognized" % node)
-                return False
-        else:
-            syntax_err(args, context=context.get_command_name())
-            return False
-        opts = ''
+        """
+        usage: standby [<node>] [<lifetime>]
+        To avoid race condition for --all option, melt all standby values into one cib replace session
+        """
+        # Parse lifetime option
+        lifetime_opt = "forever"
+        lifetime = utils.fetch_lifetime_opt(list(args), iso8601=False)
         if lifetime:
-            opts = "--lifetime='%s'" % lifetime
-        else:
-            opts = "--lifetime='forever'"
-        return utils.ext_cmd(self.node_standby % (node, "on", opts)) == 0
+            lifetime_opt = lifetime
+            args = args[:-1]
+
+        # Parse node option
+        node_list = parse_option_for_nodes(context, *args)
+        if not node_list:
+            return
+
+        # For default "forever" lifetime, under "nodes" section
+        xml_path = constants.XML_NODE_PATH
+        xml_query_path = constants.XML_NODE_QUERY_STANDBY_PATH
+        xml_query_path_oppsite = constants.XML_STATUS_QUERY_STANDBY_PATH
+        # For "reboot" lifetime, under "status" section
+        if lifetime_opt == "reboot":
+            xml_path = constants.XML_STATUS_PATH
+            xml_query_path = constants.XML_STATUS_QUERY_STANDBY_PATH
+            xml_query_path_oppsite = constants.XML_NODE_QUERY_STANDBY_PATH
+
+        cib = xmlutil.cibdump2elem()
+        xml_item_list = cib.xpath(xml_path)
+        for xml_item in xml_item_list:
+            if xml_item.get("uname") in node_list:
+                node_id = xml_item.get('id')
+                # Remove possible oppsite lifetime standby nvpair
+                item_to_del = cib.xpath(xml_query_path_oppsite.format(node_id=node_id))
+                if item_to_del:
+                    xmlutil.rmnodes(item_to_del)
+                # If the standby nvpair already exists, set and continue
+                item = cib.xpath(xml_query_path.format(node_id=node_id))
+                if item and item[0].get("value") != "on":
+                    item[0].set("value", "on")
+                    continue
+                # Create standby nvpair
+                interface_item = xml_item
+                if lifetime_opt == "reboot":
+                    res_item = xmlutil.get_set_nodes(xml_item, "transient_attributes", create=True)
+                    interface_item = res_item[0]
+                res_item = xmlutil.get_set_nodes(interface_item, "instance_attributes", create=True)
+                xmlutil.set_attr(res_item[0], "standby", "on")
+
+        orig_cib = xmlutil.cibdump2elem()
+        rc = utils.diff_and_patch(xmlutil.xml_tostring(orig_cib), xmlutil.xml_tostring(cib))
+        if not rc:
+            return False
+        for node in node_list:
+            logger.info("standby node %s", node)
 
     @command.wait
     @command.completers(compl.nodes)
-    def do_online(self, context, node=None):
-        'usage: online [<node>]'
-        if not node:
-            node = utils.this_node()
-        if not utils.is_name_sane(node):
+    def do_online(self, context, *args):
+        """
+        usage: online [<node>]
+        To avoid race condition for --all option, melt all online values into one cib replace session
+        """
+        # Parse node option
+        node_list = parse_option_for_nodes(context, *args)
+        if not node_list:
+            return
+
+        cib = xmlutil.cibdump2elem()
+        for node in node_list:
+            node_id = utils.get_nodeid_from_name(node)
+            for query_path in [constants.XML_NODE_QUERY_STANDBY_PATH, constants.XML_STATUS_QUERY_STANDBY_PATH]:
+                item = cib.xpath(query_path.format(node_id=node_id))
+                if item and item[0].get("value") != "off":
+                    item[0].set("value", "off")
+
+        orig_cib = xmlutil.cibdump2elem()
+        rc = utils.diff_and_patch(xmlutil.xml_tostring(orig_cib), xmlutil.xml_tostring(cib))
+        if not rc:
             return False
-        return utils.ext_cmd(self.node_standby % (node, "off", "--lifetime='forever'")) == 0
+        for node in node_list:
+            logger.info("online node %s", node)
 
     @command.wait
     @command.completers(compl.nodes)
@@ -369,7 +424,7 @@ class NodeMgmt(command.UI):
             if cib_elem.xpath("//node_state[@uname=\"%s\"]/@crmd" % node) == ["online"]:
                 return utils.ext_cmd(self.node_cleanup_resources % node) == 0
             elif cib_elem.xpath("//node_state[@uname=\"%s\"]/@in_ccm" % node) == ["true"]:
-                common_warn("Node is offline according to Pacemaker, but online according to corosync. First shut down node '%s'" % node)
+                logger.warning("Node is offline according to Pacemaker, but online according to corosync. First shut down node '%s'", node)
                 return False
             else:
                 return utils.ext_cmd(self.node_clear_state_118 % node) == 0
@@ -382,25 +437,27 @@ class NodeMgmt(command.UI):
         rc = True
         ec, s = utils.get_stdout("%s -p" % self.crm_node)
         if not s:
-            common_err('%s -p could not list any nodes (rc=%d)' %
-                       (self.crm_node, ec))
+            logger.error('%s -p could not list any nodes (rc=%d)', self.crm_node, ec)
             rc = False
         else:
             partition_l = s.split()
             if node in partition_l:
-                common_err("according to %s, node %s is still active" %
-                           (self.crm_node, node))
+                logger.error("according to %s, node %s is still active", self.crm_node, node)
                 rc = False
         cmd = "%s --force -R %s" % (self.crm_node, node)
         if not rc:
             if config.core.force:
-                common_info('proceeding with node %s removal' % node)
+                logger.info('proceeding with node %s removal', node)
             else:
                 return False
         ec = utils.ext_cmd(cmd)
         if ec != 0:
-            common_warn('"%s" failed, rc=%d' % (cmd, ec))
-            return False
+            node_xpath = "//nodes/node[@uname='{}']".format(node)
+            cmd = 'cibadmin --delete-all --force --xpath "{}"'.format(node_xpath)
+            rc, _, err = utils.get_stdout_stderr(cmd)
+            if rc != 0:
+                logger.error('"%s" failed, rc=%d, %s', cmd, rc, err)
+                return False
         return True
 
     @command.completers(compl.nodes)
@@ -409,15 +466,15 @@ class NodeMgmt(command.UI):
         if not utils.is_name_sane(node):
             return False
         if not xmlutil.is_our_node(node):
-            common_err("node %s not found in the CIB" % node)
+            logger.error("node %s not found in the CIB", node)
             return False
         if not self._call_delnode(node):
             return False
         if utils.ext_cmd(self.node_delete % node) != 0 or \
                 utils.ext_cmd(self.node_delete_status % node) != 0:
-            common_err("%s removed from membership, but not from CIB!" % node)
+            logger.error("%s removed from membership, but not from CIB!", node)
             return False
-        common_info("node %s deleted" % node)
+        logger.info("node %s deleted", node)
         return True
 
     @command.wait
