@@ -895,15 +895,10 @@ def wait4dc(what="", show_progress=True):
             return False
         cmd = "crmadmin -S %s" % dc
         rc, s = get_stdout(add_sudo(cmd))
-        if not s.startswith("Status"):
-            logger.warning("%s unexpected output: %s (exit code: %d)", cmd, s, rc)
+        if rc != 0:
+            logger.error("Exit code of command {} is {}".format(cmd, rc))
             return False
-        try:
-            dc_status = s.split()[-2]
-        except:
-            logger.warning("%s unexpected output: %s", cmd, s)
-            return False
-        if dc_status == "S_IDLE":
+        if re.search("S_IDLE.*ok", s):
             if output_started:
                 sys.stderr.write(" done\n")
             return True
@@ -1471,7 +1466,6 @@ def get_pcmk_version(dflt):
         if rc != 0:
             logger.error("%s exited with %d [err: %s][out: %s]", cmd, rc, err, s)
         else:
-            logger.debug("pacemaker version: [err: %s][out: %s]", err, s)
             if err.startswith("CRM Version:"):
                 version = s.split()[0]
             else:
@@ -2009,8 +2003,13 @@ def detect_aws():
     """
     Detect if in AWS
     """
-    system_version = get_stdout_or_raise_error("dmidecode -s system-version")
-    return re.search(r".*amazon.*", system_version) is not None
+    # will match on xen instances
+    xen_test = get_stdout_or_raise_error("dmidecode -s system-version").lower()
+    # will match on nitro/kvm instances
+    kvm_test = get_stdout_or_raise_error("dmidecode -s system-manufacturer").lower()
+    if "amazon" in xen_test or "amazon" in kvm_test:
+        return True
+    return False
 
 
 def detect_azure():
@@ -2487,19 +2486,19 @@ class InterfacesInfo(object):
         return False
 
 
-def check_file_content_included(source_file, target_file):
+def check_file_content_included(source_file, target_file, remote=None, source_local=False):
     """
     Check whether target_file includes contents of source_file
     """
-    if not os.path.exists(source_file):
+    if not detect_file(source_file, remote=None if source_local else remote):
         raise ValueError("File {} not exist".format(source_file))
-    if not os.path.exists(target_file):
+    if not detect_file(target_file, remote=remote):
         return False
 
-    with open(target_file, 'r') as target_fd:
-        target_data = target_fd.read()
-    with open(source_file, 'r') as source_fd:
-        source_data = source_fd.read()
+    cmd = "cat {}".format(target_file)
+    target_data = get_stdout_or_raise_error(cmd, remote=remote)
+    cmd = "cat {}".format(source_file)
+    source_data = get_stdout_or_raise_error(cmd, remote=None if source_local else remote)
     return source_data in target_data
 
 
@@ -2540,10 +2539,13 @@ class ServiceManager(object):
         elif self.remote_addr and self.remote_addr != this_node():
             prompt_msg = "Run \"{}\" on {}".format(cmd, self.remote_addr)
             rc, output, err = run_cmd_on_remote(cmd, self.remote_addr, prompt_msg)
+            # see "EXIT STATUS" in man systemctl
+            if rc > 4:
+                raise ValueError("Run \"{}\" error: {}".format(cmd, err))
         else:
             rc, output, err = get_stdout_stderr(cmd)
-        if rc != 0 and err:
-            raise ValueError("Run \"{}\" error: {}".format(cmd, err))
+            if rc != 0 and err:
+                raise ValueError("Run \"{}\" error: {}".format(cmd, err))
         return rc == 0, output
 
     @property
@@ -3075,7 +3077,13 @@ def diff_and_patch(orig_cib_str, current_cib_str):
     """
     # In cibconfig.py, _patch_cib method doesn't include status section
     # So here should make a function to handle common cases
-    cmd = "crm_diff -u -O '{}' -N '{}'".format(orig_cib_str, current_cib_str)
+    from . import tmpfiles
+    orig_cib_file = str2tmp(orig_cib_str, suffix=".xml")
+    current_cib_file = str2tmp(current_cib_str, suffix=".xml")
+    tmpfiles.add(orig_cib_file)
+    tmpfiles.add(current_cib_file)
+
+    cmd = "crm_diff -u -o '{}' -n '{}'".format(orig_cib_file, current_cib_file)
     rc, cib_diff, err = get_stdout_stderr(cmd)
     if rc == 0: # no difference
         return True
@@ -3088,4 +3096,32 @@ def diff_and_patch(orig_cib_str, current_cib_str):
         logger.error("Failed to patch")
         return False
     return True
+
+
+def read_from_file(infile):
+    """
+    Read data from file in a save way, to avoid UnicodeDecodeError
+    """
+    data = None
+    with open(infile, 'rt', encoding='utf-8', errors='replace') as f:
+        data = f.read()
+    return to_ascii(data)
+
+
+def has_dup_value(_list):
+    return _list and len(_list) != len(set(_list))
+
+
+def detect_file(_file, remote=None):
+    """
+    Detect if file exists, support both local and remote
+    """
+    rc = False
+    if not remote:
+        rc = os.path.exists(_file)
+    else:
+        cmd = "ssh {} root@{} 'test -f {}'".format(SSH_OPTION, remote, _file)
+        code, _, _ = get_stdout_stderr(cmd)
+        rc = code == 0
+    return rc
 # vim:ts=4:sw=4:et:
