@@ -13,18 +13,22 @@ from . import config
 from . import options
 from . import schema
 from . import constants
-from .msg import common_err, common_error, common_warn, common_debug, cib_parse_err, err_buf
 from . import userdir
-from .utils import add_sudo, str2file, str2tmp, get_boolean
-from .utils import get_stdout, stdout2list, crm_msec, crm_time_cmp
-from .utils import olist, get_cib_in_use, get_tempdir, to_ascii
+from .utils import add_sudo, str2file, str2tmp, get_boolean, handle_role_for_ocf_1_1
+from .utils import get_stdout, get_stdout_or_raise_error, stdout2list, crm_msec, crm_time_cmp
+from .utils import olist, get_cib_in_use, get_tempdir, to_ascii, is_boolean_true
+from . import log
+
+
+logger = log.setup_logger(__name__)
+logger_utils = log.LoggerUtils(logger)
 
 
 def xmlparse(f):
     try:
         cib_elem = etree.parse(f).getroot()
     except Exception as msg:
-        common_err("cannot parse xml: %s" % msg)
+        logger.error("cannot parse xml: %s", msg)
         return None
     return cib_elem
 
@@ -33,7 +37,7 @@ def file2cib_elem(s):
     try:
         f = open(s, 'r')
     except IOError as msg:
-        common_err(msg)
+        logger.error(msg)
         return None
     cib_elem = xmlparse(f)
     f.close()
@@ -52,7 +56,7 @@ def compressed_file_to_cib(s):
         else:
             f = open(s)
     except IOError as msg:
-        common_err(msg)
+        logger.error(msg)
         return None
     cib_elem = xmlparse(f)
     if options.regression_tests and cib_elem is None:
@@ -76,7 +80,7 @@ def sudocall(cmd):
         p.wait()
         return p.returncode, to_ascii(outp), to_ascii(errp)
     except IOError as msg:
-        common_err("running %s: %s" % (cmd, msg))
+        logger.error("running %s: %s", cmd, msg)
         return None, None, None
 
 
@@ -93,19 +97,19 @@ def cibdump2tmp():
         if outp is not None:
             return str2tmp(outp)
     except IOError as msg:
-        common_err(msg)
+        logger.error(msg)
     return None
 
 
-def cibtext2elem(cibtext):
+def text2elem(text):
     """
     Convert a text format CIB to
     an XML tree.
     """
     try:
-        return etree.fromstring(cibtext)
+        return etree.fromstring(text)
     except Exception as err:
-        cib_parse_err(err, cibtext)
+        logger_utils.text_xml_parse_err(err, text)
         return None
 
 
@@ -116,9 +120,9 @@ def cibdump2elem(section=None):
         cmd = cib_dump
     rc, outp, errp = sudocall(cmd)
     if rc == 0:
-        return cibtext2elem(outp)
+        return text2elem(outp)
     elif rc != constants.cib_no_section_rc:
-        common_error("running %s: %s" % (cmd, errp))
+        logger.error("running %s: %s", cmd, errp)
     return None
 
 
@@ -134,7 +138,7 @@ def sanity_check_nvpairs(ident, node, attr_list):
     for nvpair in node.iterchildren("nvpair"):
         n = nvpair.get("name")
         if n and n not in attr_list:
-            common_warn("%s: unknown attribute '%s'" % (ident, n))
+            logger.warning("%s: unknown attribute '%s'", ident, n)
             rc |= 1
     return rc
 
@@ -214,14 +218,24 @@ class RscState(object):
         except (IndexError, AttributeError):
             return None
 
-    def is_ms(self, ident):
+    def has_rsc_stickiness(self):
+        """
+        Check if resource-stickiness already set
+        """
+        self._init_cib()
+        if self.rsc_dflt_elem is None:
+            return False
+        attr = get_attr_in_set(self.rsc_dflt_elem, "resource-stickiness")
+        return attr is not None
+
+    def is_ms_or_promotable_clone(self, ident):
         '''
         Test if the resource is master-slave.
         '''
         rsc_node = self.rsc2node(ident)
         if rsc_node is None:
             return False
-        return is_ms(rsc_node)
+        return is_ms_or_promotable_clone(rsc_node)
 
     def rsc_clone(self, ident):
         '''
@@ -405,7 +419,7 @@ def pe2shadow(pe_file, name):
     try:
         bits = open(pe_file, 'rb').read()
     except IOError as msg:
-        common_err("open: %s" % msg)
+        logger.error("open: %s", msg)
         return False
     # decompresed if it ends with .bz2
     if pe_file.endswith(".bz2"):
@@ -414,7 +428,7 @@ def pe2shadow(pe_file, name):
     try:
         open(shadowfile(name), "wb").write(bits)
     except IOError as msg:
-        common_err("open: %s" % msg)
+        logger.error("open: %s", msg)
         return False
     return True
 
@@ -536,8 +550,10 @@ def is_group(node):
     return node.tag == "group"
 
 
-def is_ms(node):
-    return node.tag in ("master", "ms")
+def is_ms_or_promotable_clone(node):
+    is_promotable = is_boolean_true(get_attr_value(get_child_nvset_node(node), "promotable"))
+    is_ms = node.tag in ("master", "ms")
+    return is_ms or is_promotable
 
 
 def is_clone(node):
@@ -820,7 +836,7 @@ def op2list(node):
         elif name != "id":  # skip the id
             pl.append([name, node.get(name)])
     if not action:
-        common_err("op is invalid (no name)")
+        logger.error("op is invalid (no name)")
     return action, pl
 
 
@@ -1050,7 +1066,7 @@ def rename_rscref_rset(c_obj, old_id, new_id):
 def rename_rscref(c_obj, old_id, new_id):
     if rename_rscref_simple(c_obj, old_id, new_id) or \
             rename_rscref_rset(c_obj, old_id, new_id):
-        err_buf.info("modified %s from %s to %s" % (str(c_obj), old_id, new_id))
+        logger.info("modified %s from %s to %s", str(c_obj), old_id, new_id)
 
 
 def delete_rscref(c_obj, rsc_id):
@@ -1171,13 +1187,13 @@ def get_topnode(cib_elem, tag):
     "Get configuration element or create/append if there's none."
     conf_elem = cib_elem.find("configuration")
     if conf_elem is None:
-        common_err("no configuration element found!")
+        logger.error("no configuration element found!")
         return None
     if tag == "configuration":
         return conf_elem
     e = cib_elem.find("configuration/%s" % tag)
     if e is None:
-        common_debug("create configuration section %s" % tag)
+        logger.debug("create configuration section %s", tag)
         e = etree.SubElement(conf_elem, tag)
     return e
 
@@ -1250,7 +1266,7 @@ def xml_equals_unordered(a, b):
     NOTE: resource_set children SHOULD be compared with ordering.
     """
     def fail(msg):
-        common_debug("%s!=%s: %s" % (a.tag, b.tag, msg))
+        logger.debug("%s!=%s: %s", a.tag, b.tag, msg)
         return False
 
     def tagflat(x):
@@ -1376,9 +1392,9 @@ def merge_tmpl_into_prim(prim_node, tmpl_node):
 def check_id_ref(elem, id_ref):
     target = elem.xpath('.//*[@id="%s"]' % (id_ref))
     if len(target) == 0:
-        common_err("Reference not found: %s" % id_ref)
+        logger.error("Reference not found: %s", id_ref)
     elif len(target) > 1:
-        common_err("Ambiguous reference to %s" % id_ref)
+        logger.error("Ambiguous reference to %s", id_ref)
 
 
 def new(tag, **attributes):
@@ -1411,6 +1427,7 @@ def nvpair(name, value):
     """
     <nvpair name="" value="" />
     """
+    value = handle_role_for_ocf_1_1(value, name=name)
     return new("nvpair", name=name, value=value)
 
 
@@ -1478,4 +1495,68 @@ def attributes(typename, rules, values, xmlid=None, score=None):
     return e
 
 
+class CrmMonXmlParser(object):
+    """
+    Class to parse xml output of crm_mon
+    """
+    def __init__(self, peer=None):
+        """
+        Init function
+        when peer set, parse peer node's results
+        """
+        self.xml_elem = None
+        self.peer = peer
+
+    def _load(self):
+        """
+        Load xml output of crm_mon
+        """
+        output = get_stdout_or_raise_error(constants.CRM_MON_XML_OUTPUT, remote=self.peer, no_raise=True)
+        self.xml_elem = text2elem(output)
+
+    @classmethod
+    def is_node_online(cls, node):
+        """
+        Check if node online
+        """
+        cls_inst = cls()
+        cls_inst._load()
+        elem_list = cls_inst.xml_elem.xpath('//node[@name="{}" and @online="true"]'.format(node))
+        return len(elem_list) != 0
+
+    @classmethod
+    def is_resource_configured(cls, ra_type, peer=None):
+        """
+        Check if the RA configured
+        """
+        cls_inst = cls(peer=peer)
+        cls_inst._load()
+        elem_list = cls_inst.xml_elem.xpath('//resource[@resource_agent="{}"]'.format(ra_type))
+        return len(elem_list) != 0
+
+    @classmethod
+    def is_any_resource_running(cls, peer=None):
+        """
+        Check if any RA is running
+        """
+        cls_inst = cls(peer=peer)
+        cls_inst._load()
+        elem_list = cls_inst.xml_elem.xpath('//resource[@active="true"]')
+        return len(elem_list) != 0
+
+    @classmethod
+    def is_resource_started(cls, ra, peer=None):
+        """
+        Check if the RA started(in all clone instances if configured as clone)
+
+        @ra could be resource id or resource type
+        """
+        cls_inst = cls(peer=peer)
+        cls_inst._load()
+        elem_list = cls_inst.xml_elem.xpath('//resource[(@id="{ra}" or @resource_agent="{ra}") and @active="true"]'.format(ra=ra))
+        # Stopped or not exist
+        if not elem_list:
+            return False
+        # Starting will return False
+        return all([True if elem.get('role') == 'Started' else False for elem in elem_list])
 # vim:ts=4:sw=4:et:
