@@ -66,10 +66,10 @@ class TestContext(unittest.TestCase):
 
     @mock.patch('crmsh.qdevice.QDevice')
     def test_initialize_qdevice(self, mock_qdevice):
-        options = mock.Mock(qnetd_addr="node3", qdevice_port=123)
+        options = mock.Mock(qnetd_addr="node3", qdevice_port=123, stage="")
         ctx = self.ctx_inst.set_context(options)
         ctx.initialize_qdevice()
-        mock_qdevice.assert_called_once_with('node3', port=123, algo=None, tie_breaker=None, tls=None, cmds=None, mode=None)
+        mock_qdevice.assert_called_once_with('node3', port=123, algo=None, tie_breaker=None, tls=None, cmds=None, mode=None, is_stage=False)
 
     @mock.patch('crmsh.utils.fatal')
     def test_validate_sbd_option_error_together(self, mock_error):
@@ -120,14 +120,16 @@ class TestContext(unittest.TestCase):
             ctx.validate_option()
         mock_error.assert_called_once_with("Maximum number of interface is 2")
 
+    @mock.patch('crmsh.utils.has_dup_value')
     @mock.patch('crmsh.utils.fatal')
-    def test_validate_option_error_nic_dup(self, mock_error):
+    def test_validate_option_error_nic_dup(self, mock_error, mock_dup):
+        mock_dup.return_value = True
         mock_error.side_effect = SystemExit
         options = mock.Mock(nic_list=["eth2", "eth2"])
         ctx = self.ctx_inst.set_context(options)
         with self.assertRaises(SystemExit):
             ctx.validate_option()
-        mock_error.assert_called_once_with("Duplicated input")
+        mock_error.assert_called_once_with("Duplicated input for -i/--interface option")
 
     @mock.patch('logging.Logger.warning')
     @mock.patch('crmsh.bootstrap.Validation.valid_admin_ip')
@@ -277,9 +279,10 @@ class TestBootstrap(unittest.TestCase):
         mock_long.assert_called_once_with('Starting pacemaker(delaying start of sbd for 60s)')
         mock_start.assert_called_once_with('pacemaker.service', enable=False, node_list=[])
 
-    @mock.patch('crmsh.bootstrap.configure_local_ssh_key')
+    @mock.patch('crmsh.bootstrap.configure_ssh_key')
     @mock.patch('crmsh.utils.start_service')
     def test_init_ssh(self, mock_start_service, mock_config_ssh):
+        bootstrap._context = mock.Mock(node_list=[])
         bootstrap.init_ssh()
         mock_start_service.assert_called_once_with("sshd.service", enable=True)
         mock_config_ssh.assert_has_calls([
@@ -325,79 +328,75 @@ class TestBootstrap(unittest.TestCase):
         mock_nologin.assert_called_once_with("hacluster")
         mock_invoke.assert_called_once_with("usermod -s /bin/bash hacluster")
 
-    @mock.patch('crmsh.utils.this_node')
-    @mock.patch('crmsh.utils.fatal')
-    @mock.patch('crmsh.bootstrap.invoke')
-    @mock.patch('logging.Logger.info')
-    @mock.patch('os.path.exists')
-    @mock.patch('crmsh.bootstrap.key_files')
-    @mock.patch('crmsh.bootstrap.change_user_shell')
-    def test_configure_local_ssh_key_error(self, mock_change_shell, mock_key_files, mock_exists, mock_status, mock_invoke, mock_error, mock_this_node):
-        mock_key_files.return_value = {"private": "/root/.ssh/id_rsa", "public": "/root/.ssh/id_rsa.pub", "authorized": "/root/.ssh/authorized_keys"}
-        mock_exists.return_value = False
-        mock_invoke.return_value = (False, None, "error")
-        mock_this_node.return_value = "node1"
-        mock_error.side_effect = SystemExit
-
-        with self.assertRaises(SystemExit) as err:
-            bootstrap.configure_local_ssh_key("root")
-
-        mock_change_shell.assert_called_once_with("root")
-        mock_key_files.assert_called_once_with("root")
-        mock_exists.assert_called_once_with("/root/.ssh/id_rsa")
-        mock_status.assert_called_once_with("Generating SSH key for root")
-        mock_invoke.assert_called_once_with("ssh-keygen -q -f /root/.ssh/id_rsa -C 'Cluster Internal on node1' -N ''")
-        mock_error.assert_called_once_with("Failed to generate ssh key for root: error")
-
     @mock.patch('crmsh.bootstrap.append_unique')
-    @mock.patch('builtins.open', create=True)
-    @mock.patch('crmsh.bootstrap.invoke')
+    @mock.patch('crmsh.utils.get_stdout_or_raise_error')
     @mock.patch('crmsh.utils.add_su')
-    @mock.patch('crmsh.utils.this_node')
     @mock.patch('logging.Logger.info')
-    @mock.patch('os.path.exists')
+    @mock.patch('crmsh.utils.detect_file')
     @mock.patch('crmsh.bootstrap.key_files')
     @mock.patch('crmsh.bootstrap.change_user_shell')
-    def test_configure_local_ssh_key(self, mock_change_shell, mock_key_files, mock_exists, mock_status, mock_this_node, mock_su, mock_invoke, mock_open_file, mock_append):
-        bootstrap._context = mock.Mock(yes_to_all=True)
+    def test_configure_ssh_key_remote(self, mock_change_shell, mock_key_files, mock_detect, mock_info, mock_su, mock_run, mock_append):
         mock_key_files.return_value = {"private": "/test/.ssh/id_rsa", "public": "/test/.ssh/id_rsa.pub", "authorized": "/test/.ssh/authorized_keys"}
-        mock_exists.side_effect = [False, False]
-        mock_this_node.return_value = "node1"
-        mock_invoke.return_value = (True, None, None)
+        mock_detect.side_effect = [False, True]
         mock_su.return_value = "cmd with su"
 
-        bootstrap.configure_local_ssh_key("test")
+        bootstrap.configure_ssh_key("test", remote="node1")
 
         mock_change_shell.assert_called_once_with("test")
         mock_key_files.assert_called_once_with("test")
-        mock_exists.assert_has_calls([
-            mock.call("/test/.ssh/id_rsa"),
-            mock.call("/test/.ssh/authorized_keys")
+        mock_detect.assert_has_calls([
+            mock.call("/test/.ssh/id_rsa", remote="node1"),
+            mock.call("/test/.ssh/authorized_keys", remote="node1")
             ])
-        mock_status.assert_called_once_with("Generating SSH key for test")
-        mock_invoke.assert_called_once_with("cmd with su")
+        mock_info.assert_called_once_with("SSH key for test does not exist, hence generate it now")
         mock_su.assert_called_once_with("ssh-keygen -q -f /test/.ssh/id_rsa -C 'Cluster Internal on node1' -N ''", "test")
-        mock_this_node.assert_called_once_with()
-        mock_open_file.assert_called_once_with("/test/.ssh/authorized_keys", 'w')
-        mock_append.assert_called_once_with("/test/.ssh/id_rsa.pub", "/test/.ssh/authorized_keys")
+        mock_run.assert_has_calls([
+            mock.call("cmd with su", remote="node1")
+            ])
+        mock_append.assert_called_once_with("/test/.ssh/id_rsa.pub", "/test/.ssh/authorized_keys", remote="node1")
+
+    @mock.patch('crmsh.bootstrap.append_unique')
+    @mock.patch('crmsh.utils.get_stdout_or_raise_error')
+    @mock.patch('crmsh.utils.detect_file')
+    @mock.patch('crmsh.bootstrap.key_files')
+    @mock.patch('crmsh.bootstrap.change_user_shell')
+    def test_configure_ssh_key(self, mock_change_shell, mock_key_files, mock_detect, mock_run, mock_append_unique):
+        mock_key_files.return_value = {"private": "/test/.ssh/id_rsa", "public": "/test/.ssh/id_rsa.pub", "authorized": "/test/.ssh/authorized_keys"}
+        mock_detect.side_effect = [True, True, False]
+
+        bootstrap.configure_ssh_key("test")
+
+        mock_change_shell.assert_called_once_with("test")
+        mock_key_files.assert_called_once_with("test")
+        mock_detect.assert_has_calls([
+            mock.call("/test/.ssh/id_rsa", remote=None),
+            mock.call("/test/.ssh/id_rsa.pub", remote=None),
+            mock.call("/test/.ssh/authorized_keys", remote=None)
+            ])
+        mock_append_unique.assert_called_once_with("/test/.ssh/id_rsa.pub", "/test/.ssh/authorized_keys", remote=None)
+        mock_run.assert_called_once_with('touch /test/.ssh/authorized_keys', remote=None)
+
+    @mock.patch('crmsh.bootstrap.append_to_remote_file')
+    @mock.patch('crmsh.utils.check_file_content_included')
+    def test_append_unique_remote(self, mock_check, mock_append):
+        mock_check.return_value = False
+        bootstrap.append_unique("fromfile", "tofile", remote="node1", from_local=True)
+        mock_check.assert_called_once_with("fromfile", "tofile", remote="node1", source_local=True)
+        mock_append.assert_called_once_with("fromfile", "node1", "tofile")
 
     @mock.patch('crmsh.bootstrap.append')
     @mock.patch('crmsh.utils.check_file_content_included')
     def test_append_unique(self, mock_check, mock_append):
         mock_check.return_value = False
         bootstrap.append_unique("fromfile", "tofile")
-        mock_check.assert_called_once_with("fromfile", "tofile")
-        mock_append.assert_called_once_with("fromfile", "tofile")
+        mock_check.assert_called_once_with("fromfile", "tofile", remote=None, source_local=False)
+        mock_append.assert_called_once_with("fromfile", "tofile", remote=None)
 
-    @mock.patch('crmsh.utils.fatal')
-    @mock.patch('crmsh.bootstrap.invoke')
-    def test_append_to_remote_file(self, mock_invoke, mock_error):
-        mock_invoke.return_value = (False, None, "error")
-        error_string = 'Failed to append contents of fromfile to node1:\n"error"\n\n    crmsh has no way to help you to setup up passwordless ssh among nodes at this time. \n    As the hint, likely, `PasswordAuthentication` is \'no\' in /etc/ssh/sshd_config. \n    Given in this case, users must setup passwordless ssh beforehand, or change it to \'yes\' and manage passwords properly\n    '
+    @mock.patch('crmsh.utils.get_stdout_or_raise_error')
+    def test_append_to_remote_file(self, mock_run):
         bootstrap.append_to_remote_file("fromfile", "node1", "tofile")
         cmd = "cat fromfile | ssh {} root@node1 'cat >> tofile'".format(constants.SSH_OPTION)
-        mock_invoke.assert_called_once_with(cmd)
-        mock_error.assert_called_once_with(error_string)
+        mock_run.assert_called_once_with(cmd)
 
     @mock.patch('crmsh.bootstrap.invokerc')
     def test_fetch_public_key_from_remote_node_exception(self, mock_invoke):
@@ -439,7 +438,7 @@ class TestBootstrap(unittest.TestCase):
     @mock.patch('crmsh.utils.fatal')
     @mock.patch('crmsh.bootstrap.invoke')
     @mock.patch('crmsh.bootstrap.swap_public_ssh_key')
-    @mock.patch('crmsh.bootstrap.configure_local_ssh_key')
+    @mock.patch('crmsh.bootstrap.configure_ssh_key')
     @mock.patch('crmsh.utils.start_service')
     def test_join_ssh(self, mock_start_service, mock_config_ssh, mock_swap, mock_invoke, mock_error):
         bootstrap._context = mock.Mock(default_nic_list=["eth1"])
@@ -489,16 +488,37 @@ class TestBootstrap(unittest.TestCase):
         mock_key_files.return_value = {"private": "/root/.ssh/id_rsa", "public": "/root/.ssh/id_rsa.pub", "authorized": "/root/.ssh/authorized_keys"}
         mock_check_passwd.return_value = True
         mock_fetch.return_value = "file1"
+        bootstrap._context = mock.Mock(with_other_user=True)
 
-        bootstrap.swap_public_ssh_key("node1")
+        bootstrap.swap_public_ssh_key("node1", user="other")
 
-        mock_key_files.assert_called_once_with("root")
-        mock_check_passwd.assert_called_once_with("node1", "root")
-        mock_status.assert_called_once_with("Configuring SSH passwordless with root@node1")
+        mock_key_files.assert_called_once_with("other")
+        mock_check_passwd.assert_called_once_with("node1", "other")
+        mock_status.assert_called_once_with("Configuring SSH passwordless with other@node1")
         mock_append_remote.assert_called_once_with("/root/.ssh/id_rsa.pub", "node1", "/root/.ssh/authorized_keys")
-        mock_fetch.assert_called_once_with("node1", "root")
+        mock_fetch.assert_called_once_with("node1", "other")
         mock_append_unique.assert_called_once_with("file1", "/root/.ssh/authorized_keys")
 
+    @mock.patch('crmsh.utils.this_node')
+    def test_bootstrap_add_return(self, mock_this_node):
+        ctx = mock.Mock(node_list=[])
+        bootstrap.bootstrap_add(ctx)
+        mock_this_node.assert_not_called()
+
+    @mock.patch('crmsh.utils.ext_cmd_nosudo')
+    @mock.patch('logging.Logger.info')
+    @mock.patch('crmsh.utils.this_node')
+    def test_bootstrap_add(self, mock_this_node, mock_info, mock_ext):
+        ctx = mock.Mock(node_list=["node2", "node3"], nic_list=["eth1"])
+        mock_this_node.return_value = "node1"
+        bootstrap.bootstrap_add(ctx)
+        mock_info.assert_has_calls([
+            mock.call("Adding node node2 to cluster"),
+            mock.call("Running command on node2: crm cluster join -y -c node1 -i eth1"),
+            mock.call("Adding node node3 to cluster"),
+            mock.call("Running command on node3: crm cluster join -y -c node1 -i eth1")
+            ])
+
     @mock.patch('crmsh.utils.fatal')
     @mock.patch('crmsh.utils.get_stdout_stderr')
     def test_setup_passwordless_with_other_nodes_failed_fetch_nodelist(self, mock_run, mock_error):
@@ -755,16 +775,16 @@ class TestBootstrap(unittest.TestCase):
         mock_invoke.assert_called_once_with("ssh-copy-id -i /root/.ssh/id_rsa.pub root@10.10.10.123")
         mock_error.assert_called_once_with("Failed to copy ssh key: error")
 
-    @mock.patch('crmsh.bootstrap.start_qdevice_service')
     @mock.patch('crmsh.bootstrap.confirm')
     @mock.patch('crmsh.utils.is_qdevice_configured')
     @mock.patch('crmsh.utils.check_ssh_passwd_need')
     @mock.patch('logging.Logger.info')
-    def test_init_qdevice_already_configured(self, mock_status, mock_ssh, mock_qdevice_configured, mock_confirm, mock_qdevice_start):
+    def test_init_qdevice_already_configured(self, mock_status, mock_ssh, mock_qdevice_configured, mock_confirm):
         bootstrap._context = mock.Mock(qdevice_inst=self.qdevice_with_ip)
         mock_ssh.return_value = False
         mock_qdevice_configured.return_value = True
         mock_confirm.return_value = False
+        self.qdevice_with_ip.start_qdevice_service = mock.Mock()
 
         bootstrap.init_qdevice()
 
@@ -772,35 +792,27 @@ class TestBootstrap(unittest.TestCase):
         mock_ssh.assert_called_once_with("10.10.10.123")
         mock_qdevice_configured.assert_called_once_with()
         mock_confirm.assert_called_once_with("Qdevice is already configured - overwrite?")
-        mock_qdevice_start.assert_called_once_with()
+        self.qdevice_with_ip.start_qdevice_service.assert_called_once_with()
 
-    @mock.patch('crmsh.bootstrap.start_qdevice_service')
-    @mock.patch('crmsh.qdevice.QDevice.certificate_process_on_init')
-    @mock.patch('crmsh.log.LoggerUtils.status_long')
-    @mock.patch('crmsh.utils.is_qdevice_tls_on')
-    @mock.patch('crmsh.bootstrap.config_qdevice')
-    @mock.patch('crmsh.qdevice.QDevice.valid_qnetd')
     @mock.patch('crmsh.utils.is_qdevice_configured')
     @mock.patch('crmsh.utils.check_ssh_passwd_need')
     @mock.patch('logging.Logger.info')
-    def test_init_qdevice(self, mock_status, mock_ssh, mock_qdevice_configured, mock_valid_qnetd, mock_config_qdevice,
-            mock_tls, mock_status_long, mock_certificate, mock_start_qdevice):
+    def test_init_qdevice(self, mock_info, mock_ssh, mock_qdevice_configured):
         bootstrap._context = mock.Mock(qdevice_inst=self.qdevice_with_ip)
         mock_ssh.return_value = False
         mock_qdevice_configured.return_value = False
-        mock_tls.return_value = True
+        self.qdevice_with_ip.set_cluster_name = mock.Mock()
+        self.qdevice_with_ip.valid_qnetd = mock.Mock()
+        self.qdevice_with_ip.config_and_start_qdevice = mock.Mock()
 
         bootstrap.init_qdevice()
 
-        mock_status.assert_called_once_with("Configure Qdevice/Qnetd:")
+        mock_info.assert_called_once_with("Configure Qdevice/Qnetd:")
         mock_ssh.assert_called_once_with("10.10.10.123")
-        mock_valid_qnetd.assert_called_once_with()
         mock_qdevice_configured.assert_called_once_with()
-        mock_config_qdevice.assert_called_once_with()
-        mock_tls.assert_called_once_with()
-        mock_status_long.assert_called_once_with("Qdevice certification process")
-        mock_certificate.assert_called_once_with()
-        mock_start_qdevice.assert_called_once_with()
+        self.qdevice_with_ip.set_cluster_name.assert_called_once_with()
+        self.qdevice_with_ip.valid_qnetd.assert_called_once_with()
+        self.qdevice_with_ip.config_and_start_qdevice.assert_called_once_with()
 
     @mock.patch('crmsh.bootstrap.prompt_for_string')
     def test_configure_qdevice_interactive_return(self, mock_prompt):
@@ -844,99 +856,9 @@ class TestBootstrap(unittest.TestCase):
             mock.call("Whether using TLS on QDevice/QNetd (on/off/required)", default="on"),
             mock.call("Heuristics COMMAND to run with absolute path; For multiple commands, use \";\" to separate")
             ])
-        mock_qdevice.assert_called_once_with('qnetd-node', port=5403, algo='ffsplit', tie_breaker='lowest', tls='on', cmds=None, mode=None)
+        mock_qdevice.assert_called_once_with('qnetd-node', port=5403, algo='ffsplit', tie_breaker='lowest', tls='on', cmds=None, mode=None, is_stage=False)
         mock_qdevice_inst.valid_qdevice_options.assert_called_once_with()
 
-    @mock.patch('crmsh.qdevice.QDevice.start_qnetd')
-    @mock.patch('crmsh.qdevice.QDevice.enable_qnetd')
-    @mock.patch('crmsh.utils.cluster_run_cmd')
-    @mock.patch('logging.Logger.info')
-    def test_start_qdevice_service_reload(self, mock_status, mock_cluster_run, mock_enable_qnetd, mock_start_qnetd):
-        bootstrap._context = mock.Mock(qdevice_inst=self.qdevice_with_ip, qdevice_reload_policy=bootstrap.QdevicePolicy.QDEVICE_RELOAD)
-
-        bootstrap.start_qdevice_service()
-
-        mock_status.assert_has_calls([
-            mock.call("Enable corosync-qdevice.service in cluster"),
-            mock.call("Starting corosync-qdevice.service in cluster"),
-            mock.call("Enable corosync-qnetd.service on 10.10.10.123"),
-            mock.call("Starting corosync-qnetd.service on 10.10.10.123")
-            ])
-        mock_cluster_run.assert_has_calls([
-            mock.call("systemctl enable corosync-qdevice"),
-            mock.call("systemctl restart corosync-qdevice")
-            ])
-        mock_enable_qnetd.assert_called_once_with()
-        mock_start_qnetd.assert_called_once_with()
-
-    @mock.patch('crmsh.qdevice.QDevice.start_qnetd')
-    @mock.patch('crmsh.qdevice.QDevice.enable_qnetd')
-    @mock.patch('crmsh.bootstrap.wait_for_cluster')
-    @mock.patch('crmsh.utils.cluster_run_cmd')
-    @mock.patch('logging.Logger.info')
-    def test_start_qdevice_service_restart(self, mock_status, mock_cluster_run, mock_wait, mock_enable_qnetd, mock_start_qnetd):
-        bootstrap._context = mock.Mock(qdevice_inst=self.qdevice_with_ip, qdevice_reload_policy=bootstrap.QdevicePolicy.QDEVICE_RESTART)
-
-        bootstrap.start_qdevice_service()
-
-        mock_status.assert_has_calls([
-            mock.call("Enable corosync-qdevice.service in cluster"),
-            mock.call("Restarting cluster service"),
-            mock.call("Enable corosync-qnetd.service on 10.10.10.123"),
-            mock.call("Starting corosync-qnetd.service on 10.10.10.123")
-            ])
-        mock_wait.assert_called_once_with()
-        mock_cluster_run.assert_has_calls([
-            mock.call("systemctl enable corosync-qdevice"),
-            mock.call("crm cluster restart")
-            ])
-        mock_enable_qnetd.assert_called_once_with()
-        mock_start_qnetd.assert_called_once_with()
-
-    @mock.patch('crmsh.qdevice.QDevice.start_qnetd')
-    @mock.patch('crmsh.qdevice.QDevice.enable_qnetd')
-    @mock.patch('logging.Logger.warning')
-    @mock.patch('crmsh.utils.cluster_run_cmd')
-    @mock.patch('logging.Logger.info')
-    def test_start_qdevice_service_warn(self, mock_status, mock_cluster_run, mock_warn, mock_enable_qnetd, mock_start_qnetd):
-        bootstrap._context = mock.Mock(qdevice_inst=self.qdevice_with_ip, qdevice_reload_policy=bootstrap.QdevicePolicy.QDEVICE_RESTART_LATER)
-
-        bootstrap.start_qdevice_service()
-
-        mock_status.assert_has_calls([
-            mock.call("Enable corosync-qdevice.service in cluster"),
-            mock.call("Enable corosync-qnetd.service on 10.10.10.123"),
-            mock.call("Starting corosync-qnetd.service on 10.10.10.123")
-            ])
-        mock_cluster_run.assert_has_calls([
-            mock.call("systemctl enable corosync-qdevice"),
-            ])
-        mock_warn.assert_called_once_with("To use qdevice service, need to restart cluster service manually on each node")
-        mock_enable_qnetd.assert_called_once_with()
-        mock_start_qnetd.assert_called_once_with()
-
-    @mock.patch('crmsh.utils.cluster_run_cmd')
-    @mock.patch('crmsh.bootstrap.update_expected_votes')
-    @mock.patch('crmsh.log.LoggerUtils.status_long')
-    @mock.patch('crmsh.corosync.add_nodelist_from_cmaptool')
-    @mock.patch('crmsh.corosync.is_unicast')
-    @mock.patch('crmsh.qdevice.QDevice.write_qdevice_config')
-    @mock.patch('crmsh.qdevice.QDevice.remove_qdevice_db')
-    def test_config_qdevice(self, mock_remove_qdevice_db, mock_write_qdevice_config, mock_is_unicast,
-            mock_add_nodelist, mock_status_long, mock_update_votes, mock_cluster_run):
-        bootstrap._context = mock.Mock(qdevice_inst=self.qdevice_with_ip, qdevice_reload_policy=bootstrap.QdevicePolicy.QDEVICE_RELOAD)
-        mock_is_unicast.return_value = False
-
-        bootstrap.config_qdevice()
-
-        mock_remove_qdevice_db.assert_called_once_with()
-        mock_write_qdevice_config.assert_called_once_with()
-        mock_is_unicast.assert_called_once_with()
-        mock_add_nodelist.assert_called_once_with()
-        mock_status_long.assert_called_once_with("Update configuration")
-        mock_update_votes.assert_called_once_with()
-        mock_cluster_run.assert_called_once_with("crm corosync reload")
-
     @mock.patch('crmsh.utils.fatal')
     @mock.patch('crmsh.utils.is_qdevice_configured')
     def test_remove_qdevice_no_configured(self, mock_qdevice_configured, mock_error):
@@ -960,33 +882,29 @@ class TestBootstrap(unittest.TestCase):
         mock_qdevice_configured.assert_called_once_with()
         mock_confirm.assert_called_once_with("Removing QDevice service and configuration from cluster: Are you sure?")
 
+    @mock.patch('crmsh.qdevice.QDevice.remove_certification_files_on_qnetd')
+    @mock.patch('crmsh.qdevice.QDevice.remove_qdevice_db')
+    @mock.patch('crmsh.qdevice.QDevice.remove_qdevice_config')
     @mock.patch('crmsh.bootstrap.update_expected_votes')
-    @mock.patch('crmsh.qdevice.QDevice')
-    @mock.patch('crmsh.corosync.get_value')
     @mock.patch('crmsh.log.LoggerUtils.status_long')
     @mock.patch('crmsh.bootstrap.invoke')
     @mock.patch('logging.Logger.info')
-    @mock.patch('crmsh.bootstrap.evaluate_qdevice_quorum_effect')
+    @mock.patch('crmsh.qdevice.evaluate_qdevice_quorum_effect')
     @mock.patch('crmsh.utils.check_all_nodes_reachable')
     @mock.patch('crmsh.bootstrap.confirm')
     @mock.patch('crmsh.utils.is_qdevice_configured')
     def test_remove_qdevice_reload(self, mock_qdevice_configured, mock_confirm, mock_reachable, mock_evaluate,
-            mock_status, mock_invoke, mock_status_long, mock_get_value, mock_qdevice, mock_update_votes):
+            mock_status, mock_invoke, mock_status_long, mock_update_votes, mock_remove_config, mock_remove_db, mock_remove_files):
         mock_qdevice_configured.return_value = True
         mock_confirm.return_value = True
-        mock_evaluate.return_value = bootstrap.QdevicePolicy.QDEVICE_RELOAD
-        mock_get_value.return_value = "10.10.10.123"
-        mock_qdevice_inst = mock.Mock()
-        mock_qdevice.return_value = mock_qdevice_inst
-        mock_qdevice_inst.remove_qdevice_config = mock.Mock()
-        mock_qdevice_inst.remove_qdevice_db = mock.Mock()
+        mock_evaluate.return_value = qdevice.QdevicePolicy.QDEVICE_RELOAD
 
         bootstrap.remove_qdevice()
 
         mock_qdevice_configured.assert_called_once_with()
         mock_confirm.assert_called_once_with("Removing QDevice service and configuration from cluster: Are you sure?")
         mock_reachable.assert_called_once_with()
-        mock_evaluate.assert_called_once_with(bootstrap.QDEVICE_REMOVE)
+        mock_evaluate.assert_called_once_with(qdevice.QDEVICE_REMOVE)
         mock_status.assert_has_calls([
             mock.call("Disable corosync-qdevice.service"),
             mock.call("Stopping corosync-qdevice.service")
@@ -997,11 +915,9 @@ class TestBootstrap(unittest.TestCase):
             mock.call("crm cluster run 'crm corosync reload'")
             ] )
         mock_status_long.assert_called_once_with("Removing QDevice configuration from cluster")
-        mock_get_value.assert_called_once_with("quorum.device.net.host")
-        mock_qdevice.assert_called_once_with("10.10.10.123")
-        mock_qdevice_inst.remove_qdevice_config.assert_called_once_with()
-        mock_qdevice_inst.remove_qdevice_db.assert_called_once_with()
         mock_update_votes.assert_called_once_with()
+        mock_remove_config.assert_called_once_with()
+        mock_remove_db.assert_called_once_with()
 
     @mock.patch('crmsh.utils.start_service')
     @mock.patch('crmsh.qdevice.QDevice')
@@ -1057,42 +973,6 @@ class TestBootstrap(unittest.TestCase):
         self.assertEqual(res, True)
         mock_invoke.assert_called_once_with("cmd")
 
-    @mock.patch('crmsh.utils.calculate_quorate_status')
-    @mock.patch('crmsh.utils.get_quorum_votes_dict')
-    def test_evaluate_qdevice_quorum_effect_reload(self, mock_get_dict, mock_quorate):
-        mock_get_dict.return_value = {'Expected': '2', 'Total': '2'}
-        mock_quorate.return_value = True
-        res = bootstrap.evaluate_qdevice_quorum_effect(bootstrap.QDEVICE_ADD)
-        self.assertEqual(res, bootstrap.QdevicePolicy.QDEVICE_RELOAD)
-        mock_get_dict.assert_called_once_with()
-        mock_quorate.assert_called_once_with(3, 2)
-
-    @mock.patch('crmsh.xmlutil.CrmMonXmlParser.is_any_resource_running')
-    @mock.patch('crmsh.utils.calculate_quorate_status')
-    @mock.patch('crmsh.utils.get_quorum_votes_dict')
-    def test_evaluate_qdevice_quorum_effect_reload(self, mock_get_dict, mock_quorate, mock_ra_running):
-        mock_get_dict.return_value = {'Expected': '2', 'Total': '2'}
-        mock_quorate.return_value = False
-        mock_ra_running.return_value = True
-        res = bootstrap.evaluate_qdevice_quorum_effect(bootstrap.QDEVICE_REMOVE)
-        self.assertEqual(res, bootstrap.QdevicePolicy.QDEVICE_RESTART_LATER)
-        mock_get_dict.assert_called_once_with()
-        mock_quorate.assert_called_once_with(2, 1)
-        mock_ra_running.assert_called_once_with()
-
-    @mock.patch('crmsh.xmlutil.CrmMonXmlParser.is_any_resource_running')
-    @mock.patch('crmsh.utils.calculate_quorate_status')
-    @mock.patch('crmsh.utils.get_quorum_votes_dict')
-    def test_evaluate_qdevice_quorum_effect(self, mock_get_dict, mock_quorate, mock_ra_running):
-        mock_get_dict.return_value = {'Expected': '2', 'Total': '2'}
-        mock_quorate.return_value = False
-        mock_ra_running.return_value = False
-        res = bootstrap.evaluate_qdevice_quorum_effect(bootstrap.QDEVICE_REMOVE)
-        self.assertEqual(res, bootstrap.QdevicePolicy.QDEVICE_RESTART)
-        mock_get_dict.assert_called_once_with()
-        mock_quorate.assert_called_once_with(2, 1)
-        mock_ra_running.assert_called_once_with()
-
     @mock.patch('crmsh.utils.cluster_run_cmd')
     @mock.patch('os.path.isfile')
     def test_sync_files_to_disk(self, mock_isfile, mock_cluster_cmd):
@@ -1427,11 +1307,13 @@ class TestValidation(unittest.TestCase):
         mock_ext.assert_called_once_with("ssh {} node2 'crm cluster remove -y -c node1'".format(constants.SSH_OPTION))
         mock_error.assert_called_once_with("Failed to remove this node from node2")
 
+    @mock.patch('crmsh.qdevice.QDevice.remove_qdevice_db')
+    @mock.patch('crmsh.qdevice.QDevice.remove_certification_files_on_qnetd')
     @mock.patch('crmsh.utils.fatal')
     @mock.patch('crmsh.bootstrap.invokerc')
     @mock.patch('crmsh.bootstrap.stop_services')
     @mock.patch('crmsh.xmlutil.listnodes')
-    def test_remove_self_rm_failed(self, mock_list, mock_stop_service, mock_invoke, mock_error):
+    def test_remove_self_rm_failed(self, mock_list, mock_stop_service, mock_invoke, mock_error, mock_rm_files, mock_rm_db):
         mock_list.return_value = ["node1"]
         mock_invoke.return_value = False
         mock_error.side_effect = SystemExit
@@ -1468,22 +1350,25 @@ class TestValidation(unittest.TestCase):
     @mock.patch('logging.Logger.info')
     @mock.patch('crmsh.utils.service_is_active')
     def test_stop_services(self, mock_active, mock_status, mock_stop):
-        mock_active.side_effect = [True, True, True]
+        mock_active.side_effect = [True, True, True, True]
         bootstrap.stop_services(bootstrap.SERVICES_STOP_LIST)
         mock_active.assert_has_calls([
             mock.call("corosync-qdevice.service", remote_addr=None),
             mock.call("corosync.service", remote_addr=None),
-            mock.call("hawk.service", remote_addr=None)
+            mock.call("hawk.service", remote_addr=None),
+            mock.call("csync2.socket", remote_addr=None)
             ])
         mock_status.assert_has_calls([
             mock.call("Stopping the corosync-qdevice.service"),
             mock.call("Stopping the corosync.service"),
-            mock.call("Stopping the hawk.service")
+            mock.call("Stopping the hawk.service"),
+            mock.call("Stopping the csync2.socket")
             ])
         mock_stop.assert_has_calls([
             mock.call("corosync-qdevice.service", disable=True, remote_addr=None),
             mock.call("corosync.service", disable=True, remote_addr=None),
-            mock.call("hawk.service", disable=True, remote_addr=None)
+            mock.call("hawk.service", disable=True, remote_addr=None),
+            mock.call("csync2.socket", disable=True, remote_addr=None)
             ])
 
     @mock.patch('crmsh.utils.fatal')
