@@ -2,7 +2,10 @@ from __future__ import print_function
 from __future__ import unicode_literals
 # Copyright (C) 2014 Kristoffer Gronlund <kgronlund@suse.com>
 # See COPYING for license information.
-
+try:
+    from unittest import mock
+except ImportError:
+    import mock
 
 from crmsh import cibconfig
 from lxml import etree
@@ -343,25 +346,6 @@ def test_tagset():
     assert set(x.obj_id for x in elems) == set(['r1', 'r2'])
 
 
-def test_ratrace():
-    xml = '''<primitive class="ocf" id="%s" provider="pacemaker" type="Dummy"/>'''
-    factory.create_from_node(etree.fromstring(xml % ('r1')))
-    factory.create_from_node(etree.fromstring(xml % ('r2')))
-    factory.create_from_node(etree.fromstring(xml % ('r3')))
-
-    context = object()
-
-    from crmsh.ui_resource import RscMgmt
-    obj = factory.find_object('r1')
-    RscMgmt()._trace_resource(context, 'r1', obj)
-
-    obj = factory.find_object('r1')
-    ops = obj.node.xpath('./operations/op')
-    for op in ops:
-        assert op.xpath('./instance_attributes/nvpair[@name="trace_ra"]/@value') == ["1"]
-    assert set(obj.node.xpath('./operations/op/@name')) == set(['start', 'stop'])
-
-
 def test_op_role():
     xml = '''<primitive class="ocf" id="rsc2" provider="pacemaker" type="Dummy">
         <operations>
@@ -547,7 +531,9 @@ def test_existing_node_resource():
     assert ok
 
 
-def test_existing_node_resource_2():
+@mock.patch("crmsh.log.LoggerUtils.line_number")
+@mock.patch("crmsh.log.LoggerUtils.incr_lineno")
+def test_existing_node_resource_2(mock_incr, mock_line_num):
     obj = cibconfig.mkset_obj()
     assert obj is not None
 
@@ -566,7 +552,9 @@ def test_existing_node_resource_2():
     assert sorted(text.split('\n')) == sorted(text2.split('\n'))
 
 
-def test_id_collision_breakage_1():
+@mock.patch("crmsh.log.LoggerUtils.line_number")
+@mock.patch("crmsh.log.LoggerUtils.incr_lineno")
+def test_id_collision_breakage_1(mock_incr, mock_line_num):
     from crmsh import clidisplay
 
     obj = cibconfig.mkset_obj()
@@ -646,7 +634,9 @@ primitive p1 ocf:heartbeat:Dummy \
         assert original_cib == obj.repr()
 
 
-def test_id_collision_breakage_3():
+@mock.patch("crmsh.log.LoggerUtils.line_number")
+@mock.patch("crmsh.log.LoggerUtils.incr_lineno")
+def test_id_collision_breakage_3(mock_incr, mock_line_num):
     from crmsh import clidisplay
 
     obj = cibconfig.mkset_obj()
@@ -689,7 +679,9 @@ primitive node1 Dummy params fake=something
         assert original_cib == obj.repr()
 
 
-def test_id_collision_breakage_2():
+@mock.patch("crmsh.log.LoggerUtils.line_number")
+@mock.patch("crmsh.log.LoggerUtils.incr_lineno")
+def test_id_collision_breakage_2(mock_incr, mock_line_num):
     from crmsh import clidisplay
 
     obj = cibconfig.mkset_obj()
@@ -799,7 +791,9 @@ def test_bug_110():
             assert o.check_sanity() == 0
 
 
-def test_reordering_resource_sets():
+@mock.patch("crmsh.log.LoggerUtils.line_number")
+@mock.patch("crmsh.log.LoggerUtils.incr_lineno")
+def test_reordering_resource_sets(mock_incr, mock_line_num):
     """
     Can we reorder resource sets?
     """
