@@ -1,10 +1,12 @@
+import logging
 import unittest
 try:
     from unittest import mock
 except ImportError:
     import mock
-from crmsh import ocfs2, utils, ra
+from crmsh import ocfs2, utils, ra, constants
 
+logging.basicConfig(level=logging.INFO)
 
 class TestOCFS2Manager(unittest.TestCase):
     """
@@ -134,15 +136,14 @@ class TestOCFS2Manager(unittest.TestCase):
     def test_check_if_already_configured_return(self):
         self.ocfs2_inst3._check_if_already_configured()
 
-    @mock.patch('crmsh.ocfs2.err_buf')
+    @mock.patch('logging.Logger.info')
     @mock.patch('crmsh.utils.get_stdout_or_raise_error')
-    def test_check_if_already_configured(self, mock_run, mock_buf):
+    def test_check_if_already_configured(self, mock_run, mock_info):
         mock_run.return_value = "data xxx fstype=ocfs2  sss"
-        mock_buf.info = mock.Mock()
         with self.assertRaises(utils.TerminateSubCommand):
             self.ocfs2_inst2._check_if_already_configured()
         mock_run.assert_called_once_with("crm configure show")
-        mock_buf.info.assert_called_once_with("Already configured OCFS2 related resources")
+        mock_info.assert_called_once_with("Already configured OCFS2 related resources")
 
     @mock.patch('crmsh.ocfs2.OCFS2Manager._verify_devices')
     @mock.patch('crmsh.ocfs2.OCFS2Manager._check_if_already_configured')
@@ -155,20 +156,13 @@ class TestOCFS2Manager(unittest.TestCase):
         mock_configured.assert_called_once_with()
         mock_verify_devices.assert_called_once_with()
 
-    @mock.patch('crmsh.utils.compare_uuid_with_peer_dev')
-    @mock.patch('crmsh.ocfs2.OCFS2Manager._verify_packages')
-    def test_static_verify_on_join(self, mock_verify_packages, mock_uuid):
-        self.ocfs2_inst1._static_verify_on_join(["/dev/sda1"], "node1")
-        mock_verify_packages.assert_called_once_with(False)
-        mock_uuid.assert_called_once_with(["/dev/sda1"], "node1")
-
     def test_dynamic_raise_error(self):
         with self.assertRaises(ValueError) as err:
             self.ocfs2_inst2._dynamic_raise_error("error messages")
         self.assertEqual("error messages", str(err.exception))
 
     @mock.patch('crmsh.ocfs2.OCFS2Manager._dynamic_raise_error')
-    @mock.patch('crmsh.bootstrap.SBDManager.get_sbd_device_from_config')
+    @mock.patch('crmsh.sbd.SBDManager.get_sbd_device_from_config')
     @mock.patch('crmsh.utils.service_is_enabled')
     def test_check_sbd_and_ocfs2_dev(self, mock_enabled, mock_get_device, mock_error):
         mock_enabled.return_value = True
@@ -197,6 +191,29 @@ class TestOCFS2Manager(unittest.TestCase):
             mock.call("/dev/sdc2 contains a ext4 file system - Proceed anyway?")
             ])
 
+    @mock.patch('crmsh.utils.get_stdout_or_raise_error')
+    @mock.patch('crmsh.bootstrap.confirm')
+    @mock.patch('crmsh.utils.get_dev_fs_type')
+    @mock.patch('crmsh.utils.has_dev_partitioned')
+    def test_confirm_to_overwrite_ocfs2_dev_confirmed(self, mock_has_parted, mock_fstype, mock_confirm, mock_run):
+        mock_has_parted.side_effect = [True, False]
+        mock_fstype.return_value = "ext4"
+        mock_confirm.side_effect = [True, True]
+        self.ocfs2_inst3._confirm_to_overwrite_ocfs2_dev()
+        mock_has_parted.assert_has_calls([
+            mock.call("/dev/sdb2"),
+            mock.call("/dev/sdc2")
+            ])
+        mock_fstype.assert_called_once_with("/dev/sdc2")
+        mock_confirm.assert_has_calls([
+            mock.call("Found a partition table in /dev/sdb2 - Proceed anyway?"),
+            mock.call("/dev/sdc2 contains a ext4 file system - Proceed anyway?")
+            ])
+        mock_run.assert_has_calls([
+            mock.call("wipefs -a /dev/sdb2"),
+            mock.call("wipefs -a /dev/sdc2")
+            ])
+
     @mock.patch('crmsh.ocfs2.OCFS2Manager._dynamic_raise_error')
     @mock.patch('crmsh.utils.has_stonith_running')
     def test_dynamic_verify_error(self, mock_has_stonith, mock_error):
@@ -227,7 +244,7 @@ class TestOCFS2Manager(unittest.TestCase):
 
     @mock.patch('crmsh.utils.get_stdout_or_raise_error')
     @mock.patch('crmsh.corosync.get_value')
-    @mock.patch('crmsh.bootstrap.status_long')
+    @mock.patch('crmsh.log.LoggerUtils.status_long')
     def test_mkfs(self, mock_long, mock_get_value, mock_run):
         mock_get_value.return_value = "hacluster"
         self.ocfs2_inst3._mkfs("/dev/sdb2")
@@ -249,7 +266,7 @@ class TestOCFS2Manager(unittest.TestCase):
     @mock.patch('crmsh.utils.gen_unused_id')
     @mock.patch('crmsh.utils.get_all_vg_name')
     @mock.patch('crmsh.utils.get_stdout_or_raise_error')
-    @mock.patch('crmsh.bootstrap.status_long')
+    @mock.patch('crmsh.log.LoggerUtils.status_long')
     def test_create_lv(self, mock_long, mock_run, mock_all_vg, mock_unused, mock_pe_num):
         mock_all_vg.return_value = []
         mock_unused.return_value = "vg1"
@@ -357,34 +374,42 @@ class TestOCFS2Manager(unittest.TestCase):
         mock_fs.assert_called_once_with()
 
     @mock.patch('crmsh.ocfs2.OCFS2Manager._config_resource_stack_lvm2')
+    @mock.patch('crmsh.utils.set_property')
+    @mock.patch('crmsh.utils.get_property')
     @mock.patch('crmsh.utils.all_exist_id')
     @mock.patch('crmsh.ocfs2.OCFS2Manager._dynamic_verify')
-    @mock.patch('crmsh.bootstrap.status')
-    def test_init_ocfs2_lvm2(self, mock_status, mock_dynamic_verify, mock_all_id, mock_lvm2):
+    @mock.patch('logging.Logger.info')
+    def test_init_ocfs2_lvm2(self, mock_status, mock_dynamic_verify, mock_all_id, mock_get, mock_set, mock_lvm2):
         mock_all_id.return_value = []
+        mock_get.return_value = None
         self.ocfs2_inst7.mount_point = "/data"
         self.ocfs2_inst7.target_device = "/dev/vg1/lv1"
         self.ocfs2_inst7.init_ocfs2()
         mock_status.assert_has_calls([
             mock.call("Configuring OCFS2"),
-            mock.call("  OCFS2 device /dev/vg1/lv1 mounted on /data")
+            mock.call('  \'no-quorum-policy\' is changed to "freeze"'),
+            mock.call('  OCFS2 device %s mounted on %s', '/dev/vg1/lv1', '/data')
             ])
         mock_dynamic_verify.assert_called_once_with()
         mock_all_id.assert_called_once_with()
         mock_lvm2.assert_called_once_with()
 
     @mock.patch('crmsh.ocfs2.OCFS2Manager._config_resource_stack_ocfs2_along')
+    @mock.patch('crmsh.utils.set_property')
+    @mock.patch('crmsh.utils.get_property')
     @mock.patch('crmsh.utils.all_exist_id')
     @mock.patch('crmsh.ocfs2.OCFS2Manager._dynamic_verify')
-    @mock.patch('crmsh.bootstrap.status')
-    def test_init_ocfs2(self, mock_status, mock_dynamic_verify, mock_all_id, mock_ocfs2):
+    @mock.patch('logging.Logger.info')
+    def test_init_ocfs2(self, mock_status, mock_dynamic_verify, mock_all_id, mock_get, mock_set, mock_ocfs2):
         mock_all_id.return_value = []
+        mock_get.return_value = None
         self.ocfs2_inst3.mount_point = "/data"
         self.ocfs2_inst3.target_device = "/dev/sda1"
         self.ocfs2_inst3.init_ocfs2()
         mock_status.assert_has_calls([
             mock.call("Configuring OCFS2"),
-            mock.call("  OCFS2 device /dev/sda1 mounted on /data")
+            mock.call('  \'no-quorum-policy\' is changed to "freeze"'),
+            mock.call('  OCFS2 device %s mounted on %s', '/dev/sda1', '/data')
             ])
         mock_dynamic_verify.assert_called_once_with()
         mock_all_id.assert_called_once_with()
@@ -416,41 +441,28 @@ params directory="/srv/clusterfs" fstype=ocfs2 device="/dev/sda2"
         self.assertEqual(res, "/dev/sda2")
         mock_run.assert_called_once_with("crm configure show", remote="node1")
 
-    def test_get_device_list_on_join_return(self):
-        res = self.ocfs2_inst3._get_device_list_on_join("/dev/sda2", "node1")
-        self.assertEqual(res, ["/dev/sda2"])
-
-    @mock.patch('crmsh.utils.get_stdout_or_raise_error')
-    def test_get_device_list_on_join(self, mock_run):
-        mock_run.return_value = """
-LV       VG       Attr       LSize Pool Origin Data%  Meta%  Move Log Cpy%Sync Convert Devices
-ocfs2-lv ocfs2-vg -wi-ao---- 5.99g                                                     /dev/sda2(0)
-ocfs2-lv ocfs2-vg -wi-ao---- 5.99g                                                     /dev/sda3(0)
-        """
-        res = self.ocfs2_inst3._get_device_list_on_join("/dev/vg1/lv1", "node1", True)
-        self.assertEqual(res, ["/dev/sda2", "/dev/sda3"])
-        mock_run.assert_called_once_with("lvs /dev/vg1/lv1 -o +devices", remote="node1")
-
     @mock.patch('crmsh.ocfs2.OCFS2Manager._find_target_on_join')
     def test_join_ocfs2_return(self, mock_find):
         mock_find.return_value = None
         self.ocfs2_inst3.join_ocfs2("node1")
         mock_find.assert_called_once_with("node1")
 
-    @mock.patch('crmsh.ocfs2.OCFS2Manager._static_verify_on_join')
-    @mock.patch('crmsh.ocfs2.OCFS2Manager._get_device_list_on_join')
-    @mock.patch('crmsh.utils.has_resource_configured')
-    @mock.patch('crmsh.bootstrap.status_long')
+    @mock.patch('crmsh.utils.compare_uuid_with_peer_dev')
+    @mock.patch('crmsh.utils.is_dev_a_plain_raw_disk_or_partition')
+    @mock.patch('crmsh.ocfs2.OCFS2Manager._verify_packages')
+    @mock.patch('crmsh.xmlutil.CrmMonXmlParser.is_resource_configured')
+    @mock.patch('crmsh.log.LoggerUtils.status_long')
     @mock.patch('crmsh.ocfs2.OCFS2Manager._find_target_on_join')
-    def test_join_ocfs2(self, mock_find, mock_long, mock_configured, mock_get_devlist, mock_static_verify):
+    def test_join_ocfs2(self, mock_find, mock_long, mock_configured, mock_verify_packages, mock_is_mapper, mock_compare):
         mock_find.return_value = "/dev/sda2"
         mock_configured.return_value = False
-        mock_get_devlist.return_value = ["/dev/sda2"]
+        mock_is_mapper.return_value = True
         self.ocfs2_inst3.join_ocfs2("node1")
         mock_find.assert_called_once_with("node1")
-        mock_configured.assert_called_once_with("ocf::heartbeat:lvmlockd", "node1")
-        mock_get_devlist.assert_called_once_with("/dev/sda2", "node1", False)
-        mock_static_verify.assert_called_once_with(["/dev/sda2"], "node1", False)
+        mock_configured.assert_called_once_with(constants.LVMLOCKD_RA, "node1")
+        mock_verify_packages.assert_called_once_with(False)
+        mock_is_mapper.assert_called_once_with("/dev/sda2", "node1")
+        mock_compare.assert_called_once_with(["/dev/sda2"], "node1")
 
     @mock.patch('crmsh.ocfs2.OCFS2Manager._static_verify')
     def test_verify_ocfs2(self, mock_static_verify):
